from orkg.utils import NamespacedClient, query_params


class ComparisonsClient(NamespacedClient):

    @query_params("contribution_ids", "title", "description", "reference")
    def add(self, params=None):
        if 'contribution_ids' not in params or 'title' not in params:
            raise ValueError('at least contribution_ids and title should be provided')

        for contribution_id in params['contribution_ids']:
            contribution_resource = self.client.resources.by_id(id=contribution_id).content

            if 'Contribution' not in contribution_resource['classes']:
                raise ValueError('this ID is not a contribution: ' + str(contribution_id))

        description = params['description'] if 'description' in params else ''
        reference = params['reference'] if 'reference' in params else ''
        contribution_ids = ','.join(params['contribution_ids'])

        comparison_resp = self.client.resources.add(label=params['title'], classes=['Comparison'])
        comparison_id = comparison_resp.content['id']
        description_id = self.client.literals.add(label=description).content['id']
        reference_id = self.client.literals.add(label=reference).content['id']
        url_id = self.client.literals.add(label='?contributions=' + contribution_ids).content['id']

        self.client.statements.add(subject_id=comparison_id, predicate_id='description', object_id=description_id)
        self.client.statements.add(subject_id=comparison_id, predicate_id='url', object_id=url_id)
        self.client.statements.add(subject_id=comparison_id, predicate_id='reference', object_id=reference_id)

        print('Created successfully, comparison id: ' + comparison_id)

        return self.client.wrap_response(comparison_resp)



import hammock
from .resources import ResourcesClient
from .predicates import PredicatesClient
from .classes import ClassesClient
from .literals import LiteralsClient
from .stats import StatsClient
from .statements import StatementsClient
from .papers import PapersClient
from .comparisons import ComparisonsClient
from .contributions import ContributionsClient
from .objects import ObjectsClient
from .dummy import DummyClient
from orkg.out import OrkgResponse
from typing import Union


class ORKG(object):

    def __init__(self, host=None, simcomp_host=None, creds=None, **kwargs):
        if host is not None and not host.startswith('http'):
            raise ValueError('host must begin with http or https')
        self.host = host if host is not None else 'http://127.0.0.1:8080'
        if self.host[-1] == '/':
            self.host = self.host[:-1]
        if simcomp_host is not None and not simcomp_host.startswith('http'):
            raise ValueError('simcomp host must begin with http or https')
        self.simcomp_host = simcomp_host
        self.core = hammock.Hammock(self.host)
        if self.simcomp_available:
            self.simcomp = hammock.Hammock(self.simcomp_host)
        self.token = None
        if creds is not None and len(creds) == 2:
            self.__authenticate(creds[0], creds[1])
        self.backend = self.core.api
        self.resources = ResourcesClient(self)
        self.predicates = PredicatesClient(self)
        self.classes = ClassesClient(self)
        self.literals = LiteralsClient(self)
        self.stats = StatsClient(self)
        self.statements = StatementsClient(self)
        self.papers = PapersClient(self)
        self.comparisons = ComparisonsClient(self)
        self.contributions = ContributionsClient(self)
        self.objects = ObjectsClient(self)
        self.dummy = DummyClient(self)

    @property
    def simcomp_available(self):
        return self.simcomp_host is not None

    def __authenticate(self, email, password):
        data = {
            'username': email,
            'grant_type': 'password',
            'client_id': 'orkg-client',
            'password': password
        }
        resp = self.core.oauth.token.POST(data=data, headers={'Authorization': 'Basic b3JrZy1jbGllbnQ6c2VjcmV0'})
        if resp.status_code == 200:
            self.token = resp.json()['access_token']
        else:
            raise IOError(f"Please check the credentials provided!, got the error {resp.content}")

    def ping(self):
        pass  # TODO: see how to ping ORKG host to know if it is alive

    # FIXME: remove after system stabling
    def pagination_activated(self) -> bool:
        self.backend._append_slash = True
        return 'pageable' in self.backend.resources.GET().json()

    def wrap_response(self, response=None, status_code: str = None, content: Union[list, dict] = None,
                      url: str = None) -> OrkgResponse:
        is_paged = self.pagination_activated
        return OrkgResponse(response=response, status_code=status_code, content=content, url=url, paged=is_paged)



from unittest import TestCase
from orkg import ORKG


class TestOut(TestCase):
    """
    Some test scenarios might need to be adjusted to the content of the running ORKG instance
    """

    orkg = ORKG()

    def test_new_pagination_wrapper(self):
        res = self.orkg.resources.get()
        if self.orkg.pagination_activated():
            self.assertIsNotNone(res.pageable, 'pageable info is not None')
            self.assertTrue('content' not in res.content, 'content extraction performed correctly')
        else:
            self.assertIsNone(res.pageable, 'pageable info is None because back-end is not up to date')
            self.assertWarns(DeprecationWarning)

    def test_new_pagination_wrapper_on_none_get_call(self):
        label = "test"
        res = self.orkg.resources.add(label=label)
        self.assertTrue(res.succeeded)
        if self.orkg.pagination_activated():
            self.assertIsNone(res.pageable, 'pageable is not available in POST calls')
            self.assertTrue('content' not in res.content, 'content extraction performed correctly')
        else:
            self.assertIsNone(res.pageable, 'pageable is not available in POST calls even if the backend is up-to-date')
            self.assertWarns(DeprecationWarning)


